@@ -14,11 +14,11 @@ Geo::Coder::Free - Provides a geocoding functionality using free databases
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 SYNOPSIS
 
@@ -28,7 +28,7 @@ our $VERSION = '0.09';
     my $location = $geocoder->geocode(location => 'Ramsgate, Kent, UK');
 
     # Use a local download of http://results.openaddresses.io/
-    my $openaddr_geocoder = Geo::Coder::Freee->new(openaddr => $ENV{'OPENADDR_HOME'});
+    my $openaddr_geocoder = Geo::Coder::Free->new(openaddr => $ENV{'OPENADDR_HOME'});
     $location = $openaddr_geocoder->geocode(location => '1600 Pennsylvania Avenue NW, Washington DC, USA');
 
 =head1 DESCRIPTION
@@ -49,6 +49,8 @@ The cgi-bin directory contains a simple DIY geocoding website:
 Takes one optional parameter, openaddr, which is the base directory of
 the OpenAddresses data downloaded from L<http://results.openaddresses.io>.
 
+The database also will include data from Who's On First L<https://whosonfirst.org>.
+
 Takes one optional parameter, directory,
 which tells the library where to find the MaxMind and GeoNames files admin1db, admin2.db and cities.[sql|csv.gz].
 If that parameter isn't given, the module will attempt to find the databases, but that can't be guaranteed.
@@ -188,6 +190,9 @@ must apply in writing for a licence for use from Nigel Horne at `<njh at nigelho
 This product includes GeoLite2 data created by MaxMind, available from
 L<http://www.maxmind.com>.
 
+This product includes data from Who's on First.
+See L<https://github.com/whosonfirst-data/whosonfirst-data/blob/master/LICENSE.md> for licensing information.
+
 =cut
 
 1;
